@@ -19,8 +19,11 @@
 import copy
 import json
 import os
+import re
 import warnings
-from typing import Any, Dict, Tuple, Union
+from typing import Any, Dict, Optional, Tuple, Union
+
+from packaging import version
 
 from . import __version__
 from .file_utils import (
@@ -28,6 +31,7 @@ from .file_utils import (
     PushToHubMixin,
     cached_path,
     copy_func,
+    get_list_of_files,
     hf_bucket_url,
     is_offline_mode,
     is_remote_url,
@@ -37,6 +41,8 @@ from .utils import logging
 
 
 logger = logging.get_logger(__name__)
+FULL_CONFIGURATION_FILE = "config.json"
+_re_configuration_file = re.compile(r"config\.(.*)\.json")
 
 
 class PretrainedConfig(PushToHubMixin):
@@ -44,180 +50,185 @@ class PretrainedConfig(PushToHubMixin):
     Base class for all configuration classes. Handles a few parameters common to all models' configurations as well as
     methods for loading/downloading/saving configurations.
 
-    Note:
-        A configuration file can be loaded and saved to disk. Loading the configuration file and using this file to
-        initialize a model does **not** load the model weights. It only affects the model's configuration.
+    <Tip>
 
-    Class attributes (overridden by derived classes)
+    A configuration file can be loaded and saved to disk. Loading the configuration file and using this file to
+    initialize a model does **not** load the model weights. It only affects the model's configuration.
 
-        - **model_type** (:obj:`str`) -- An identifier for the model type, serialized into the JSON file, and used to
-          recreate the correct object in :class:`~transformers.AutoConfig`.
-        - **is_composition** (:obj:`bool`) -- Whether the config class is composed of multiple sub-configs. In this
-          case the config has to be initialized from two or more configs of type
-          :class:`~transformers.PretrainedConfig` like: :class:`~transformers.EncoderDecoderConfig` or
-          :class:`~RagConfig`.
-        - **keys_to_ignore_at_inference** (:obj:`List[str]`) -- A list of keys to ignore by default when looking at
-          dictionary outputs of the model during inference.
-        - **attribute_map** (:obj:`Dict[str, str]`) -- A dict that maps model specific attribute names to the
-          standardized naming of attributes.
+    </Tip>
 
-    Common attributes (present in all subclasses)
+    Class attributes (overridden by derived classes):
 
-        - **vocab_size** (:obj:`int`) -- The number of tokens in the vocabulary, which is also the first dimension of
-          the embeddings matrix (this attribute may be missing for models that don't have a text modality like ViT).
-        - **hidden_size** (:obj:`int`) -- The hidden size of the model.
-        - **num_attention_heads** (:obj:`int`) -- The number of attention heads used in the multi-head attention layers
-          of the model.
-        - **num_hidden_layers** (:obj:`int`) -- The number of blocks in the model.
+    - **model_type** (`str`) -- An identifier for the model type, serialized into the JSON file, and used to recreate
+      the correct object in [`~transformers.AutoConfig`].
+    - **is_composition** (`bool`) -- Whether the config class is composed of multiple sub-configs. In this case the
+      config has to be initialized from two or more configs of type [`~transformers.PretrainedConfig`] like:
+      [`~transformers.EncoderDecoderConfig`] or [`~RagConfig`].
+    - **keys_to_ignore_at_inference** (`List[str]`) -- A list of keys to ignore by default when looking at dictionary
+      outputs of the model during inference.
+    - **attribute_map** (`Dict[str, str]`) -- A dict that maps model specific attribute names to the standardized
+      naming of attributes.
 
-    Args:
-        name_or_path (:obj:`str`, `optional`, defaults to :obj:`""`):
-            Store the string that was passed to :func:`~transformers.PreTrainedModel.from_pretrained` or
-            :func:`~transformers.TFPreTrainedModel.from_pretrained` as ``pretrained_model_name_or_path`` if the
-            configuration was created with such a method.
-        output_hidden_states (:obj:`bool`, `optional`, defaults to :obj:`False`):
+    Common attributes (present in all subclasses):
+
+    - **vocab_size** (`int`) -- The number of tokens in the vocabulary, which is also the first dimension of the
+      embeddings matrix (this attribute may be missing for models that don't have a text modality like ViT).
+    - **hidden_size** (`int`) -- The hidden size of the model.
+    - **num_attention_heads** (`int`) -- The number of attention heads used in the multi-head attention layers of the
+      model.
+    - **num_hidden_layers** (`int`) -- The number of blocks in the model.
+
+    Arg:
+        name_or_path (`str`, _optional_, defaults to `""`):
+            Store the string that was passed to [`PreTrainedModel.from_pretrained`] or
+            [`TFPreTrainedModel.from_pretrained`] as `pretrained_model_name_or_path` if the configuration was created
+            with such a method.
+        output_hidden_states (`bool`, _optional_, defaults to `False`):
             Whether or not the model should return all hidden-states.
-        output_attentions (:obj:`bool`, `optional`, defaults to :obj:`False`):
+        output_attentions (`bool`, _optional_, defaults to `False`):
             Whether or not the model should returns all attentions.
-        return_dict (:obj:`bool`, `optional`, defaults to :obj:`True`):
-            Whether or not the model should return a :class:`~transformers.file_utils.ModelOutput` instead of a plain
-            tuple.
-        is_encoder_decoder (:obj:`bool`, `optional`, defaults to :obj:`False`):
+        return_dict (`bool`, _optional_, defaults to `True`):
+            Whether or not the model should return a [`~transformers.file_utils.ModelOutput`] instead of a plain tuple.
+        is_encoder_decoder (`bool`, _optional_, defaults to `False`):
             Whether the model is used as an encoder/decoder or not.
-        is_decoder (:obj:`bool`, `optional`, defaults to :obj:`False`):
+        is_decoder (`bool`, _optional_, defaults to `False`):
             Whether the model is used as decoder or not (in which case it's used as an encoder).
-        add_cross_attention (:obj:`bool`, `optional`, defaults to :obj:`False`):
+        cross_attention_hidden_size** (`bool`, _optional_):
+            The hidden size of the cross-attention layer in case the model is used as a decoder in an encoder-decoder
+            setting and the cross-attention hidden dimension differs from `self.config.hidden_size`.
+        add_cross_attention (`bool`, _optional_, defaults to `False`):
             Whether cross-attention layers should be added to the model. Note, this option is only relevant for models
-            that can be used as decoder models within the `:class:~transformers.EncoderDecoderModel` class, which
-            consists of all models in ``AUTO_MODELS_FOR_CAUSAL_LM``.
-        tie_encoder_decoder (:obj:`bool`, `optional`, defaults to :obj:`False`):
+            that can be used as decoder models within the [`EncoderDecoderModel`] class, which consists of all models
+            in `AUTO_MODELS_FOR_CAUSAL_LM`.
+        tie_encoder_decoder (`bool`, _optional_, defaults to `False`):
             Whether all encoder weights should be tied to their equivalent decoder weights. This requires the encoder
             and decoder model to have the exact same parameter names.
-        prune_heads (:obj:`Dict[int, List[int]]`, `optional`, defaults to :obj:`{}`):
+        prune_heads (`Dict[int, List[int]]`, _optional_, defaults to `{}`):
             Pruned heads of the model. The keys are the selected layer indices and the associated values, the list of
             heads to prune in said layer.
 
-            For instance ``{1: [0, 2], 2: [2, 3]}`` will prune heads 0 and 2 on layer 1 and heads 2 and 3 on layer 2.
-        chunk_size_feed_forward (:obj:`int`, `optional`, defaults to :obj:`0`):
-            The chunk size of all feed forward layers in the residual attention blocks. A chunk size of :obj:`0` means
-            that the feed forward layer is not chunked. A chunk size of n means that the feed forward layer processes
-            :obj:`n` < sequence_length embeddings at a time. For more information on feed forward chunking, see `How
-            does Feed Forward Chunking work? <../glossary.html#feed-forward-chunking>`__ .
+            For instance `{1: [0, 2], 2: [2, 3]}` will prune heads 0 and 2 on layer 1 and heads 2 and 3 on layer 2.
+        chunk_size_feed_forward (`int`, _optional_, defaults to `0`):
+            The chunk size of all feed forward layers in the residual attention blocks. A chunk size of `0` means that
+            the feed forward layer is not chunked. A chunk size of n means that the feed forward layer processes `n` <
+            sequence_length embeddings at a time. For more information on feed forward chunking, see [How does Feed
+            Forward Chunking work?](../glossary.html#feed-forward-chunking).
 
-    Parameters for sequence generation
+        > Parameters for sequence generation
 
-        - **max_length** (:obj:`int`, `optional`, defaults to 20) -- Maximum length that will be used by default in the
-          :obj:`generate` method of the model.
-        - **min_length** (:obj:`int`, `optional`, defaults to 10) -- Minimum length that will be used by default in the
-          :obj:`generate` method of the model.
-        - **do_sample** (:obj:`bool`, `optional`, defaults to :obj:`False`) -- Flag that will be used by default in the
-          :obj:`generate` method of the model. Whether or not to use sampling ; use greedy decoding otherwise.
-        - **early_stopping** (:obj:`bool`, `optional`, defaults to :obj:`False`) -- Flag that will be used by default
-          in the :obj:`generate` method of the model. Whether to stop the beam search when at least ``num_beams``
-          sentences are finished per batch or not.
-        - **num_beams** (:obj:`int`, `optional`, defaults to 1) -- Number of beams for beam search that will be used by
-          default in the :obj:`generate` method of the model. 1 means no beam search.
-        - **num_beam_groups** (:obj:`int`, `optional`, defaults to 1) -- Number of groups to divide :obj:`num_beams`
-          into in order to ensure diversity among different groups of beams that will be used by default in the
-          :obj:`generate` method of the model. 1 means no group beam search.
-        - **diversity_penalty** (:obj:`float`, `optional`, defaults to 0.0) -- Value to control diversity for group
-          beam search. that will be used by default in the :obj:`generate` method of the model. 0 means no diversity
-          penalty. The higher the penalty, the more diverse are the outputs.
-        - **temperature** (:obj:`float`, `optional`, defaults to 1) -- The value used to module the next token
-          probabilities that will be used by default in the :obj:`generate` method of the model. Must be strictly
-          positive.
-        - **top_k** (:obj:`int`, `optional`, defaults to 50) -- Number of highest probability vocabulary tokens to keep
-          for top-k-filtering that will be used by default in the :obj:`generate` method of the model.
-        - **top_p** (:obj:`float`, `optional`, defaults to 1) -- Value that will be used by default in the
-          :obj:`generate` method of the model for ``top_p``. If set to float < 1, only the most probable tokens with
-          probabilities that add up to ``top_p`` or higher are kept for generation.
-        - **repetition_penalty** (:obj:`float`, `optional`, defaults to 1) -- Parameter for repetition penalty that
-          will be used by default in the :obj:`generate` method of the model. 1.0 means no penalty.
-        - **length_penalty** (:obj:`float`, `optional`, defaults to 1) -- Exponential penalty to the length that will
-          be used by default in the :obj:`generate` method of the model.
-        - **no_repeat_ngram_size** (:obj:`int`, `optional`, defaults to 0) -- Value that will be used by default in the
-          :obj:`generate` method of the model for ``no_repeat_ngram_size``. If set to int > 0, all ngrams of that size
-          can only occur once.
-        - **encoder_no_repeat_ngram_size** (:obj:`int`, `optional`, defaults to 0) -- Value that will be used by
-          default in the :obj:`generate` method of the model for ``encoder_no_repeat_ngram_size``. If set to int > 0,
-          all ngrams of that size that occur in the ``encoder_input_ids`` cannot occur in the ``decoder_input_ids``.
-        - **bad_words_ids** (:obj:`List[int]`, `optional`) -- List of token ids that are not allowed to be generated
-          that will be used by default in the :obj:`generate` method of the model. In order to get the tokens of the
-          words that should not appear in the generated text, use :obj:`tokenizer.encode(bad_word,
-          add_prefix_space=True)`.
-        - **num_return_sequences** (:obj:`int`, `optional`, defaults to 1) -- Number of independently computed returned
-          sequences for each element in the batch that will be used by default in the :obj:`generate` method of the
-          model.
-        - **output_scores** (:obj:`bool`, `optional`, defaults to :obj:`False`) -- Whether the model should return the
-          logits when used for generation
-        - **return_dict_in_generate** (:obj:`bool`, `optional`, defaults to :obj:`False`) -- Whether the model should
-          return a :class:`~transformers.file_utils.ModelOutput` instead of a :obj:`torch.LongTensor`
-        - **forced_bos_token_id** (:obj:`int`, `optional`) -- The id of the token to force as the first generated token
-          after the :obj:`decoder_start_token_id`. Useful for multilingual models like :doc:`mBART
-          <../model_doc/mbart>` where the first generated token needs to be the target language token.
-        - **forced_eos_token_id** (:obj:`int`, `optional`) -- The id of the token to force as the last generated token
-          when :obj:`max_length` is reached.
-        - **remove_invalid_values** (:obj:`bool`, `optional`) -- Whether to remove possible `nan` and `inf` outputs of
-          the model to prevent the generation method to crash. Note that using ``remove_invalid_values`` can slow down
-          generation.
+        max_length (`int`, _optional_, defaults to 20):
+            Maximum length that will be used by default in the `generate` method of the model.
+        min_length (`int`, _optional_, defaults to 10):
+            Minimum length that will be used by default in the `generate` method of the model.
+        do_sample (`bool`, _optional_, defaults to `False`):
+            Flag that will be used by default in the `generate` method of the model. Whether or not to use sampling ;
+            use greedy decoding otherwise.
+        early_stopping (`bool`, _optional_, defaults to `False`):
+            Flag that will be used by default in the `generate` method of the model. Whether to stop the beam search
+            when at least `num_beams` sentences are finished per batch or not.
+        num_beams (`int`, _optional_, defaults to 1):
+            Number of beams for beam search that will be used by default in the `generate` method of the model. 1 means
+            no beam search.
+        num_beam_groups (`int`, _optional_, defaults to 1):
+            Number of groups to divide `num_beams` into in order to ensure diversity among different groups of beams
+            that will be used by default in the `generate` method of the model. 1 means no group beam search.
+        diversity_penalty (`float`, _optional_, defaults to 0.0):
+            Value to control diversity for group beam search. that will be used by default in the `generate` method of
+            the model. 0 means no diversity penalty. The higher the penalty, the more diverse are the outputs.
+        temperature (`float`, _optional_, defaults to 1):
+            The value used to module the next token probabilities that will be used by default in the `generate` method
+            of the model. Must be strictly positive.
+        top_k (`int`, _optional_, defaults to 50):
+            Number of highest probability vocabulary tokens to keep for top-k-filtering that will be used by default in
+            the `generate` method of the model.
+        top_p (`float`, _optional_, defaults to 1):
+            Value that will be used by default in the `generate` method of the model for `top_p`. If set to float < 1,
+            only the most probable tokens with probabilities that add up to `top_p` or higher are kept for generation.
+        repetition_penalty (`float`, _optional_, defaults to 1):
+            Parameter for repetition penalty that will be used by default in the `generate` method of the model. 1.0
+            means no penalty.
+        length_penalty (`float`, _optional_, defaults to 1):
+            Exponential penalty to the length that will be used by default in the `generate` method of the model.
+        no_repeat_ngram_size (`int`, _optional_, defaults to 0) -- Value that will be used by default in the
+            `generate` method of the model for `no_repeat_ngram_size`. If set to int > 0, all ngrams of that size can
+            only occur once.
+        encoder_no_repeat_ngram_size (`int`, _optional_, defaults to 0) -- Value that will be used by
+            default in the `generate` method of the model for `encoder_no_repeat_ngram_size`. If set to int > 0, all
+            ngrams of that size that occur in the `encoder_input_ids` cannot occur in the `decoder_input_ids`.
+        bad_words_ids (`List[int]`, _optional_):
+            List of token ids that are not allowed to be generated that will be used by default in the `generate`
+            method of the model. In order to get the tokens of the words that should not appear in the generated text,
+            use `tokenizer.encode(bad_word, add_prefix_space=True)`.
+        num_return_sequences (`int`, _optional_, defaults to 1):
+            Number of independently computed returned sequences for each element in the batch that will be used by
+            default in the `generate` method of the model.
+        output_scores (`bool`, _optional_, defaults to `False`):
+            Whether the model should return the logits when used for generation.
+        return_dict_in_generate (`bool`, _optional_, defaults to `False`):
+            Whether the model should return a [`~transformers.file_utils.ModelOutput`] instead of a `torch.LongTensor`.
+        forced_bos_token_id (`int`, _optional_):
+            The id of the token to force as the first generated token after the `decoder_start_token_id`. Useful for
+            multilingual models like [mBART](../model_doc/mbart) where the first generated token needs to be the target
+            language token.
+        forced_eos_token_id (`int`, _optional_):
+            The id of the token to force as the last generated token when `max_length` is reached.
+        remove_invalid_values (`bool`, _optional_):
+            Whether to remove possible _nan_ and _inf_ outputs of the model to prevent the generation method to crash.
+            Note that using `remove_invalid_values` can slow down generation.
 
+        > Parameters for fine-tuning tasks
 
-    Parameters for fine-tuning tasks
+        architectures (`List[str]`, _optional_): Model architectures that can be used with the model pretrained weights.
+        finetuning_task (`str`, _optional_):
+            Name of the task used to fine-tune the model. This can be used when converting from an original (TensorFlow
+            or PyTorch) checkpoint.
+        id2label (`Dict[int, str]`, _optional_):
+            A map from index (for instance prediction index, or target index) to label.
+        label2id (`Dict[str, int]`, _optional_): A map from label to index for the model.
+        num_labels (`int`, _optional_):
+            Number of labels to use in the last layer added to the model, typically for a classification task.
+        task_specific_params (`Dict[str, Any]`, _optional_):
+            Additional keyword arguments to store for the current task.
+        problem_type (`str`, _optional_):
+            Problem type for `XxxForSequenceClassification` models. Can be one of `"regression"`,
+            `"single_label_classification"` or `"multi_label_classification"`.
 
-        - **architectures** (:obj:`List[str]`, `optional`) -- Model architectures that can be used with the model
-          pretrained weights.
-        - **finetuning_task** (:obj:`str`, `optional`) -- Name of the task used to fine-tune the model. This can be
-          used when converting from an original (TensorFlow or PyTorch) checkpoint.
-        - **id2label** (:obj:`Dict[int, str]`, `optional`) -- A map from index (for instance prediction index, or
-          target index) to label.
-        - **label2id** (:obj:`Dict[str, int]`, `optional`) -- A map from label to index for the model.
-        - **num_labels** (:obj:`int`, `optional`) -- Number of labels to use in the last layer added to the model,
-          typically for a classification task.
-        - **task_specific_params** (:obj:`Dict[str, Any]`, `optional`) -- Additional keyword arguments to store for the
-          current task.
-        - **problem_type** (:obj:`str`, `optional`) -- Problem type for :obj:`XxxForSequenceClassification` models. Can
-          be one of (:obj:`"regression"`, :obj:`"single_label_classification"`, :obj:`"multi_label_classification"`).
-          Please note that this parameter is only available in the following models: `AlbertForSequenceClassification`,
-          `BertForSequenceClassification`, `BigBirdForSequenceClassification`, `ConvBertForSequenceClassification`,
-          `DistilBertForSequenceClassification`, `ElectraForSequenceClassification`, `FunnelForSequenceClassification`,
-          `LongformerForSequenceClassification`, `MobileBertForSequenceClassification`,
-          `ReformerForSequenceClassification`, `RobertaForSequenceClassification`,
-          `SqueezeBertForSequenceClassification`, `XLMForSequenceClassification` and `XLNetForSequenceClassification`.
+        > Parameters linked to the tokenizer
 
-    Parameters linked to the tokenizer
+        tokenizer_class (`str`, _optional_):
+            The name of the associated tokenizer class to use (if none is set, will use the tokenizer associated to the
+            model by default).
+        prefix (`str`, _optional_):
+            A specific prompt that should be added at the beginning of each text before calling the model.
+        bos_token_id (`int`, _optional_): The id of the _beginning-of-stream_ token.
+        pad_token_id (`int`, _optional_): The id of the _padding_ token.
+        eos_token_id (`int`, _optional_): The id of the _end-of-stream_ token.
+        decoder_start_token_id (`int`, _optional_):
+            If an encoder-decoder model starts decoding with a different token than _bos_, the id of that token.
+        sep_token_id (`int`, _optional_): The id of the _separation_ token.
 
-        - **tokenizer_class** (:obj:`str`, `optional`) -- The name of the associated tokenizer class to use (if none is
-          set, will use the tokenizer associated to the model by default).
-        - **prefix** (:obj:`str`, `optional`) -- A specific prompt that should be added at the beginning of each text
-          before calling the model.
-        - **bos_token_id** (:obj:`int`, `optional`)) -- The id of the `beginning-of-stream` token.
-        - **pad_token_id** (:obj:`int`, `optional`)) -- The id of the `padding` token.
-        - **eos_token_id** (:obj:`int`, `optional`)) -- The id of the `end-of-stream` token.
-        - **decoder_start_token_id** (:obj:`int`, `optional`)) -- If an encoder-decoder model starts decoding with a
-          different token than `bos`, the id of that token.
-        - **sep_token_id** (:obj:`int`, `optional`)) -- The id of the `separation` token.
+        > PyTorch specific parameters
 
-    PyTorch specific parameters
+        torchscript (`bool`, _optional_, defaults to `False`):
+            Whether or not the model should be used with Torchscript.
+        tie_word_embeddings (`bool`, _optional_, defaults to `True`):
+            Whether the model's input and output word embeddings should be tied. Note that this is only relevant if the
+            model has a output word embedding layer.
+        torch_dtype (`str`, _optional_):
+            The `dtype` of the weights. This attribute can be used to initialize the model to a non-default `dtype`
+            (which is normally `float32`) and thus allow for optimal storage allocation. For example, if the saved
+            model is `float16`, ideally we want to load it back using the minimal amount of memory needed to load
+            `float16` weights. Since the config object is stored in plain text, this attribute contains just the
+            floating type string without the `torch.` prefix. For example, for `torch.float16` ``torch_dtype` is the
+            `"float16"` string.
 
-        - **torchscript** (:obj:`bool`, `optional`, defaults to :obj:`False`) -- Whether or not the model should be
-          used with Torchscript.
-        - **tie_word_embeddings** (:obj:`bool`, `optional`, defaults to :obj:`True`) -- Whether the model's input and
-          output word embeddings should be tied. Note that this is only relevant if the model has a output word
-          embedding layer.
-        - **torch_dtype** (:obj:`str`, `optional`) -- The :obj:`dtype` of the weights. This attribute can be used to
-          initialize the model to a non-default ``dtype`` (which is normally ``float32``) and thus allow for optimal
-          storage allocation. For example, if the saved model is ``float16``, ideally we want to load it back using the
-          minimal amount of memory needed to load ``float16`` weights. Since the config object is stored in plain text,
-          this attribute contains just the floating type string without the ``torch.`` prefix. For example, for
-          ``torch.float16`` ``torch_dtype`` is the ``"float16"`` string.
+            This attribute is currently not being used during model loading time, but this may change in the future
+            versions. But we can already start preparing for the future by saving the dtype with save_pretrained.
 
-          This attribute is currently not being used during model loading time, but this may change in the future
-          versions. But we can already start preparing for the future by saving the dtype with save_pretrained.
+        > TensorFlow specific parameters
 
-    TensorFlow specific parameters
-
-        - **use_bfloat16** (:obj:`bool`, `optional`, defaults to :obj:`False`) -- Whether or not the model should use
-          BFloat16 scalars (only used by some TensorFlow models).
+        use_bfloat16 (`bool`, _optional_, defaults to `False`):
+            Whether or not the model should use BFloat16 scalars (only used by some TensorFlow models).
     """
     model_type: str = ""
     is_composition: bool = False
@@ -249,6 +260,7 @@ class PretrainedConfig(PushToHubMixin):
         # Is decoder is used in encoder-decoder models to differentiate encoder from decoder
         self.is_encoder_decoder = kwargs.pop("is_encoder_decoder", False)
         self.is_decoder = kwargs.pop("is_decoder", False)
+        self.cross_attention_hidden_size = kwargs.pop("cross_attention_hidden_size", None)
         self.add_cross_attention = kwargs.pop("add_cross_attention", False)
         self.tie_encoder_decoder = kwargs.pop("tie_encoder_decoder", False)
 
@@ -314,7 +326,7 @@ class PretrainedConfig(PushToHubMixin):
         allowed_problem_types = ("regression", "single_label_classification", "multi_label_classification")
         if self.problem_type is not None and self.problem_type not in allowed_problem_types:
             raise ValueError(
-                f"The config parameter `problem_type` was not understood: received {self.problem_type}"
+                f"The config parameter `problem_type` was not understood: received {self.problem_type} "
                 "but only 'regression', 'single_label_classification' and 'multi_label_classification' are valid."
             )
 
@@ -332,7 +344,7 @@ class PretrainedConfig(PushToHubMixin):
         self.transformers_version = kwargs.pop("transformers_version", None)
 
         # Deal with gradient checkpointing
-        if kwargs.get("gradient_checkpointing", True):
+        if kwargs.get("gradient_checkpointing", False):
             warnings.warn(
                 "Passing `gradient_checkpointing` to a config initialization is deprecated and will be removed in v5 "
                 "Transformers. Using `model.gradient_checkpointing_enable()` instead, or if you are using the "
@@ -532,15 +544,23 @@ class PretrainedConfig(PushToHubMixin):
             local_files_only = True
 
         pretrained_model_name_or_path = str(pretrained_model_name_or_path)
-        if os.path.isdir(pretrained_model_name_or_path):
-            config_file = os.path.join(pretrained_model_name_or_path, CONFIG_NAME)
-        elif os.path.isfile(pretrained_model_name_or_path) or is_remote_url(pretrained_model_name_or_path):
+        if os.path.isfile(pretrained_model_name_or_path) or is_remote_url(pretrained_model_name_or_path):
             config_file = pretrained_model_name_or_path
         else:
-            config_file = hf_bucket_url(
-                pretrained_model_name_or_path, filename=CONFIG_NAME, revision=revision, mirror=None
+            configuration_file = get_configuration_file(
+                pretrained_model_name_or_path,
+                revision=revision,
+                use_auth_token=use_auth_token,
+                local_files_only=local_files_only,
             )
 
+            if os.path.isdir(pretrained_model_name_or_path):
+                config_file = os.path.join(pretrained_model_name_or_path, configuration_file)
+            else:
+                config_file = hf_bucket_url(
+                    pretrained_model_name_or_path, filename=configuration_file, revision=revision, mirror=None
+                )
+
         try:
             # Load from URL or cache if already cached
             resolved_config_file = cached_path(
@@ -560,7 +580,8 @@ class PretrainedConfig(PushToHubMixin):
             logger.error(err)
             msg = (
                 f"Can't load config for '{pretrained_model_name_or_path}'. Make sure that:\n\n"
-                f"- '{pretrained_model_name_or_path}' is a correct model identifier listed on 'https://huggingface.co/models'\n\n"
+                f"- '{pretrained_model_name_or_path}' is a correct model identifier listed on 'https://huggingface.co/models'\n"
+                f"  (make sure '{pretrained_model_name_or_path}' is not a path to a local directory with something else, in that case)\n\n"
                 f"- or '{pretrained_model_name_or_path}' is the correct path to a directory containing a {CONFIG_NAME} file\n\n"
             )
 
@@ -791,6 +812,56 @@ class PretrainedConfig(PushToHubMixin):
             d["torch_dtype"] = str(d["torch_dtype"]).split(".")[1]
 
 
+def get_configuration_file(
+    path_or_repo: Union[str, os.PathLike],
+    revision: Optional[str] = None,
+    use_auth_token: Optional[Union[bool, str]] = None,
+    local_files_only: bool = False,
+) -> str:
+    """
+    Get the configuration file to use for this version of transformers.
+
+    Args:
+        path_or_repo (:obj:`str` or :obj:`os.PathLike`):
+            Can be either the id of a repo on huggingface.co or a path to a `directory`.
+        revision(:obj:`str`, `optional`, defaults to :obj:`"main"`):
+            The specific model version to use. It can be a branch name, a tag name, or a commit id, since we use a
+            git-based system for storing models and other artifacts on huggingface.co, so ``revision`` can be any
+            identifier allowed by git.
+        use_auth_token (:obj:`str` or `bool`, `optional`):
+            The token to use as HTTP bearer authorization for remote files. If :obj:`True`, will use the token
+            generated when running :obj:`transformers-cli login` (stored in :obj:`~/.huggingface`).
+        local_files_only (:obj:`bool`, `optional`, defaults to :obj:`False`):
+            Whether or not to only rely on local files and not to attempt to download any files.
+
+    Returns:
+        :obj:`str`: The configuration file to use.
+    """
+    # Inspect all files from the repo/folder.
+    all_files = get_list_of_files(
+        path_or_repo, revision=revision, use_auth_token=use_auth_token, local_files_only=local_files_only
+    )
+    configuration_files_map = {}
+    for file_name in all_files:
+        search = _re_configuration_file.search(file_name)
+        if search is not None:
+            v = search.groups()[0]
+            configuration_files_map[v] = file_name
+    available_versions = sorted(configuration_files_map.keys())
+
+    # Defaults to FULL_CONFIGURATION_FILE and then try to look at some newer versions.
+    configuration_file = FULL_CONFIGURATION_FILE
+    transformers_version = version.parse(__version__)
+    for v in available_versions:
+        if version.parse(v) <= transformers_version:
+            configuration_file = configuration_files_map[v]
+        else:
+            # No point going further since the versions are sorted.
+            break
+
+    return configuration_file
+
+
 PretrainedConfig.push_to_hub = copy_func(PretrainedConfig.push_to_hub)
 PretrainedConfig.push_to_hub.__doc__ = PretrainedConfig.push_to_hub.__doc__.format(
     object="config", object_class="AutoConfig", object_files="configuration file"
