@@ -50,6 +50,7 @@ from .configuration_visual_bert import VisualBertConfig
 logger = logging.get_logger(__name__)
 
 _CONFIG_FOR_DOC = "VisualBertConfig"
+_CHECKPOINT_FOR_DOC = "uclanlp/visualbert-vqa-coco-pre"
 
 VISUAL_BERT_PRETRAINED_MODEL_ARCHIVE_LIST = [
     "uclanlp/visualbert-vqa",
@@ -122,7 +123,7 @@ class VisualBertEmbeddings(nn.Module):
             inputs_embeds = self.word_embeddings(input_ids)
 
         if token_type_ids is None:
-            token_type_ids = torch.zeros(input_shape, dtype=torch.long, device=self.input_embeds.device)
+            token_type_ids = torch.zeros(input_shape, dtype=torch.long, device=self.position_ids.device)
 
         token_type_embeddings = self.token_type_embeddings(token_type_ids)
 
@@ -173,7 +174,7 @@ class VisualBertEmbeddings(nn.Module):
                 if visual_position_embeddings.size(1) != visual_embeds.size(1):
                     if visual_position_embeddings.size(1) < visual_embeds.size(1):
                         raise ValueError(
-                            f"Visual position embeddings length: {visual_position_embeddings.size(1)}"
+                            f"Visual position embeddings length: {visual_position_embeddings.size(1)} "
                             f"should be the same as `visual_embeds` length: {visual_embeds.size(1)}"
                         )
                     visual_position_embeddings = visual_position_embeddings[:, : visual_embeds.size(1), :]
@@ -243,7 +244,7 @@ class VisualBertSelfAttention(nn.Module):
             attention_scores = attention_scores + attention_mask
 
         # Normalize the attention scores to probabilities.
-        attention_probs = nn.Softmax(dim=-1)(attention_scores)
+        attention_probs = nn.functional.softmax(attention_scores, dim=-1)
 
         # This is actually dropping out entire tokens to attend to, which might
         # seem a bit unusual, but is taken from the original Transformer paper.
@@ -397,6 +398,7 @@ class VisualBertEncoder(nn.Module):
         super().__init__()
         self.config = config
         self.layer = nn.ModuleList([VisualBertLayer(config) for _ in range(config.num_hidden_layers)])
+        self.gradient_checkpointing = False
 
     def forward(
         self,
@@ -416,7 +418,7 @@ class VisualBertEncoder(nn.Module):
 
             layer_head_mask = head_mask[i] if head_mask is not None else None
 
-            if getattr(self.config, "gradient_checkpointing", False) and self.training:
+            if self.gradient_checkpointing and self.training:
 
                 def create_custom_forward(module):
                     def custom_forward(*inputs):
@@ -531,6 +533,7 @@ class VisualBertPreTrainedModel(PreTrainedModel):
 
     config_class = VisualBertConfig
     base_model_prefix = "visual_bert"
+    supports_gradient_checkpointing = True
     _keys_to_ignore_on_load_missing = [r"position_ids"]
 
     def _init_weights(self, module):
@@ -546,6 +549,10 @@ class VisualBertPreTrainedModel(PreTrainedModel):
         if isinstance(module, nn.Linear) and module.bias is not None:
             module.bias.data.zero_()
 
+    def _set_gradient_checkpointing(self, module, value=False):
+        if isinstance(module, VisualBertEncoder):
+            module.gradient_checkpointing = value
+
 
 @dataclass
 class VisualBertForPreTrainingOutput(ModelOutput):
@@ -694,7 +701,8 @@ class VisualBertModel(VisualBertPreTrainedModel):
         if self.bypass_transformer:
             self.additional_layer = VisualBertLayer(config)
 
-        self.init_weights()
+        # Initialize weights and apply final processing
+        self.post_init()
 
     def get_input_embeddings(self):
         return self.embeddings.word_embeddings
@@ -733,27 +741,27 @@ class VisualBertModel(VisualBertPreTrainedModel):
 
         Example::
 
-            >>> # Assumption: `get_visual_embeddings(image)` gets the visual embeddings of the image.
-            >>> from transformers import BertTokenizer, VisualBertModel
-            >>> import torch
+            # Assumption: `get_visual_embeddings(image)` gets the visual embeddings of the image.
+            from transformers import BertTokenizer, VisualBertModel
+            import torch
 
-            >>> tokenizer = BertTokenizer.from_pretrained('bert-base-uncased')
-            >>> model = VisualBertModel.from_pretrained('uclanlp/visualbert-vqa-coco-pre')
+            tokenizer = BertTokenizer.from_pretrained('bert-base-uncased')
+            model = VisualBertModel.from_pretrained('uclanlp/visualbert-vqa-coco-pre')
 
-            >>> inputs = tokenizer("The capital of France is Paris.", return_tensors="pt")
-            >>> visual_embeds = get_visual_embeddings(image).unsqueeze(0)
-            >>> visual_token_type_ids = torch.ones(visual_embeds.shape[:-1], dtype=torch.long) #example
-            >>> visual_attention_mask = torch.ones(visual_embeds.shape[:-1], dtype=torch.float)
+            inputs = tokenizer("The capital of France is Paris.", return_tensors="pt")
+            visual_embeds = get_visual_embeddings(image).unsqueeze(0)
+            visual_token_type_ids = torch.ones(visual_embeds.shape[:-1], dtype=torch.long)
+            visual_attention_mask = torch.ones(visual_embeds.shape[:-1], dtype=torch.float)
 
-            >>> inputs.update({{
-            ...     "visual_embeds": visual_embeds,
-            ...     "visual_token_type_ids": visual_token_type_ids,
-            ...     "visual_attention_mask": visual_attention_mask
-            ... }})
+            inputs.update({
+                "visual_embeds": visual_embeds,
+                "visual_token_type_ids": visual_token_type_ids,
+                "visual_attention_mask": visual_attention_mask
+            })
 
-            >>> outputs = model(**inputs)
+            outputs = model(**inputs)
 
-            >>> last_hidden_states = outputs.last_hidden_state
+            last_hidden_states = outputs.last_hidden_state
         """
 
         output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
@@ -766,35 +774,35 @@ class VisualBertModel(VisualBertPreTrainedModel):
             raise ValueError("You cannot specify both input_ids and inputs_embeds at the same time")
         elif input_ids is not None:
             input_shape = input_ids.size()
-            batch_size, seq_length = input_shape
         elif inputs_embeds is not None:
             input_shape = inputs_embeds.size()[:-1]
-            batch_size, seq_length = input_shape
         else:
             raise ValueError("You have to specify either input_ids or inputs_embeds")
 
-        if visual_embeds is None:
-            raise ValueError(
-                f"`visual_embeds` can not be of type {type(visual_embeds)} when using a VisualBert Model."
-            )
-
+        batch_size, seq_length = input_shape
         device = input_ids.device if input_ids is not None else inputs_embeds.device
 
-        visual_input_shape = visual_embeds.size()[:-1]
+        if visual_embeds is not None:
+            visual_input_shape = visual_embeds.size()[:-1]
 
         if attention_mask is None:
             attention_mask = torch.ones(input_shape, device=device)
 
-        if visual_attention_mask is None:
+        if visual_embeds is not None and visual_attention_mask is None:
             visual_attention_mask = torch.ones(visual_input_shape, device=device)
 
         # We can provide a self-attention mask of dimensions [batch_size, from_seq_length, to_seq_length]
         # ourselves in which case we just need to make it broadcastable to all heads.
+        if visual_embeds is not None:
+            combined_attention_mask = torch.cat((attention_mask, visual_attention_mask), dim=-1)
+            extended_attention_mask: torch.Tensor = self.get_extended_attention_mask(
+                combined_attention_mask, [batch_size, input_shape + visual_input_shape], device
+            )
 
-        combined_attention_mask = torch.cat((attention_mask, visual_attention_mask), dim=-1)
-        extended_attention_mask: torch.Tensor = self.get_extended_attention_mask(
-            combined_attention_mask, [batch_size, input_shape + visual_input_shape], device
-        )
+        else:
+            extended_attention_mask: torch.Tensor = self.get_extended_attention_mask(
+                attention_mask, [batch_size, input_shape], device
+            )
 
         # Prepare head mask if needed
         # 1.0 in head_mask indicate we keep the head
@@ -870,7 +878,8 @@ class VisualBertForPreTraining(VisualBertPreTrainedModel):
         self.visual_bert = VisualBertModel(config)
         self.cls = VisualBertPreTrainingHeads(config)
 
-        self.init_weights()
+        # Initialize weights and apply final processing
+        self.post_init()
 
     def get_output_embeddings(self):
         return self.cls.predictions.decoder
@@ -914,31 +923,31 @@ class VisualBertForPreTraining(VisualBertPreTrainedModel):
 
         Example::
 
-            >>> # Assumption: `get_visual_embeddings(image)` gets the visual embeddings of the image in the batch.
-            >>> from transformers import BertTokenizer, VisualBertForPreTraining
+            # Assumption: `get_visual_embeddings(image)` gets the visual embeddings of the image in the batch.
+            from transformers import BertTokenizer, VisualBertForPreTraining
 
-            >>> tokenizer = BertTokenizer.from_pretrained('bert-base-uncased')
-            >>> model = VisualBertForPreTraining.from_pretrained('uclanlp/visualbert-vqa-coco-pre')
+            tokenizer = BertTokenizer.from_pretrained('bert-base-uncased')
+            model = VisualBertForPreTraining.from_pretrained('uclanlp/visualbert-vqa-coco-pre')
 
-            >>> inputs = tokenizer("The capital of France is {mask}.", return_tensors="pt")
-            >>> visual_embeds = get_visual_embeddings(image).unsqueeze(0)
-            >>> visual_token_type_ids = torch.ones(visual_embeds.shape[:-1], dtype=torch.long) #example
-            >>> visual_attention_mask = torch.ones(visual_embeds.shape[:-1], dtype=torch.float)
+            inputs = tokenizer("The capital of France is {mask}.", return_tensors="pt")
+            visual_embeds = get_visual_embeddings(image).unsqueeze(0)
+            visual_token_type_ids = torch.ones(visual_embeds.shape[:-1], dtype=torch.long)
+            visual_attention_mask = torch.ones(visual_embeds.shape[:-1], dtype=torch.float)
 
-            >>> inputs.update({{
-            ...     "visual_embeds": visual_embeds,
-            ...     "visual_token_type_ids": visual_token_type_ids,
-            ...     "visual_attention_mask": visual_attention_mask
-            ... }})
-            >>> max_length  = inputs["input_ids"].shape[-1]+visual_embeds.shape[-2]
-            >>> labels = tokenizer("The capital of France is Paris.", return_tensors="pt", padding="max_length", max_length=max_length)["input_ids"]
-            >>> sentence_image_labels = torch.tensor(1).unsqueeze(0) # Batch_size
+            inputs.update({
+                "visual_embeds": visual_embeds,
+                "visual_token_type_ids": visual_token_type_ids,
+                "visual_attention_mask": visual_attention_mask
+            })
+            max_length  = inputs["input_ids"].shape[-1]+visual_embeds.shape[-2]
+            labels = tokenizer("The capital of France is Paris.", return_tensors="pt", padding="max_length", max_length=max_length)["input_ids"]
+            sentence_image_labels = torch.tensor(1).unsqueeze(0) # Batch_size
 
 
-            >>> outputs = model(**inputs, labels=labels, sentence_image_labels=sentence_image_labels)
-            >>> loss = outputs.loss
-            >>> prediction_logits = outputs.prediction_logits
-            >>> seq_relationship_logits = outputs.seq_relationship_logits
+            outputs = model(**inputs, labels=labels, sentence_image_labels=sentence_image_labels)
+            loss = outputs.loss
+            prediction_logits = outputs.prediction_logits
+            seq_relationship_logits = outputs.seq_relationship_logits
         """
         return_dict = return_dict if return_dict is not None else self.config.use_return_dict
 
@@ -966,7 +975,7 @@ class VisualBertForPreTraining(VisualBertPreTrainedModel):
             total_size = attention_mask.size(-1) + visual_attention_mask.size(-1)
             if labels.size(-1) != total_size:
                 raise ValueError(
-                    f"The labels provided should have same sequence length as total attention mask."
+                    f"The labels provided should have same sequence length as total attention mask. "
                     f"Found labels with sequence length {labels.size(-1)}, expected {total_size}."
                 )
 
@@ -979,7 +988,7 @@ class VisualBertForPreTraining(VisualBertPreTrainedModel):
             total_size = attention_mask.size(-1) + visual_attention_mask.size(-1)
             if labels.size(-1) != total_size:
                 raise ValueError(
-                    f"The labels provided should have same sequence length as total attention mask."
+                    f"The labels provided should have same sequence length as total attention mask. "
                     f"Found labels with sequence length {labels.size(-1)}, expected {total_size}."
                 )
 
@@ -1014,7 +1023,8 @@ class VisualBertForMultipleChoice(VisualBertPreTrainedModel):
         self.dropout = nn.Dropout(config.hidden_dropout_prob)
         self.cls = nn.Linear(config.hidden_size, 1)
 
-        self.init_weights()
+        # Initialize weights and apply final processing
+        self.post_init()
 
     @add_start_docstrings_to_model_forward(
         VISUAL_BERT_INPUTS_DOCSTRING.format("batch_size, num_choices, sequence_length")
@@ -1047,37 +1057,38 @@ class VisualBertForMultipleChoice(VisualBertPreTrainedModel):
 
         Example::
 
-            >>> from transformers import BertTokenizer, VisualBertForMultipleChoice
-            >>> import torch
+            # Assumption: `get_visual_embeddings(image)` gets the visual embeddings of the image in the batch.
+            from transformers import BertTokenizer, VisualBertForMultipleChoice
+            import torch
 
-            >>> tokenizer = BertTokenizer.from_pretrained('bert-base-uncased')
-            >>> model = VisualBertForMultipleChoice.from_pretrained('uclanlp/visualbert-vcr')
+            tokenizer = BertTokenizer.from_pretrained('bert-base-uncased')
+            model = VisualBertForMultipleChoice.from_pretrained('uclanlp/visualbert-vcr')
 
-            >>> prompt = "In Italy, pizza served in formal settings, such as at a restaurant, is presented unsliced."
-            >>> choice0 = "It is eaten with a fork and a knife."
-            >>> choice1 = "It is eaten while held in the hand."
+            prompt = "In Italy, pizza served in formal settings, such as at a restaurant, is presented unsliced."
+            choice0 = "It is eaten with a fork and a knife."
+            choice1 = "It is eaten while held in the hand."
 
-            >>> visual_embeds = get_visual_embeddings(image)
-            >>> # (batch_size, num_choices, visual_seq_length, visual_embedding_dim)
-            >>> visual_embeds = visual_embeds.expand(1, 2, *visual_embeds.shape)
-            >>> visual_token_type_ids = torch.ones(visual_embeds.shape[:-1], dtype=torch.long)
-            >>> visual_attention_mask = torch.ones(visual_embeds.shape[:-1], dtype=torch.float)
+            visual_embeds = get_visual_embeddings(image)
+            # (batch_size, num_choices, visual_seq_length, visual_embedding_dim)
+            visual_embeds = visual_embeds.expand(1, 2, *visual_embeds.shape)
+            visual_token_type_ids = torch.ones(visual_embeds.shape[:-1], dtype=torch.long)
+            visual_attention_mask = torch.ones(visual_embeds.shape[:-1], dtype=torch.float)
 
-            >>> labels = torch.tensor(0).unsqueeze(0)  # choice0 is correct (according to Wikipedia ;)), batch size 1
+            labels = torch.tensor(0).unsqueeze(0)  # choice0 is correct (according to Wikipedia ;)), batch size 1
 
-            >>> encoding = tokenizer([[prompt, prompt], [choice0, choice1]], return_tensors='pt', padding=True)
-            >>> # batch size is 1
-            >>> inputs_dict = {{k: v.unsqueeze(0) for k,v in encoding.items()}}
-            >>> inputs_dict.update({{
-            ... visual_embeds=visual_embeds,
-            ... visual_attention_mask=visual_attention_mask,
-            ... visual_token_type_ids=visual_token_type_ids,
-            ... labels=labels
-            ... }})
-            >>> outputs = model(**inputs_dict)
+            encoding = tokenizer([[prompt, prompt], [choice0, choice1]], return_tensors='pt', padding=True)
+            # batch size is 1
+            inputs_dict = {k: v.unsqueeze(0) for k,v in encoding.items()}
+            inputs_dict.update({
+                "visual_embeds": visual_embeds,
+                "visual_attention_mask": visual_attention_mask,
+                "visual_token_type_ids": visual_token_type_ids,
+                "labels": labels
+            })
+            outputs = model(**inputs_dict)
 
-            >>> loss = outputs.loss
-            >>> logits = outputs.logits
+            loss = outputs.loss
+            logits = outputs.logits
         """
         return_dict = return_dict if return_dict is not None else self.config.use_return_dict
         num_choices = input_ids.shape[1] if input_ids is not None else inputs_embeds.shape[1]
@@ -1163,7 +1174,8 @@ class VisualBertForQuestionAnswering(VisualBertPreTrainedModel):
         self.dropout = nn.Dropout(config.hidden_dropout_prob)
         self.cls = nn.Linear(config.hidden_size, config.num_labels)
 
-        self.init_weights()
+        # Initialize weights and apply final processing
+        self.post_init()
 
     @add_start_docstrings_to_model_forward(VISUAL_BERT_INPUTS_DOCSTRING.format("batch_size, sequence_length"))
     @replace_return_docstrings(output_type=SequenceClassifierOutput, config_class=_CONFIG_FOR_DOC)
@@ -1193,30 +1205,30 @@ class VisualBertForQuestionAnswering(VisualBertPreTrainedModel):
 
         Example::
 
-            >>> # Assumption: `get_visual_embeddings(image)` gets the visual embeddings of the image in the batch.
-            >>> from transformers import BertTokenizer, VisualBertForQuestionAnswering
-            >>> import torch
+            # Assumption: `get_visual_embeddings(image)` gets the visual embeddings of the image in the batch.
+            from transformers import BertTokenizer, VisualBertForQuestionAnswering
+            import torch
 
-            >>> tokenizer = BertTokenizer.from_pretrained('bert-base-uncased')
-            >>> model = VisualBertForQuestionAnswering.from_pretrained('uclanlp/visualbert-vqa')
+            tokenizer = BertTokenizer.from_pretrained('bert-base-uncased')
+            model = VisualBertForQuestionAnswering.from_pretrained('uclanlp/visualbert-vqa')
 
-            >>> text = "Who is eating the apple?"
-            >>> inputs = tokenizer(text, return_tensors='pt')
-            >>> visual_embeds = get_visual_embeddings(image).unsqueeze(0)
-            >>> visual_token_type_ids = torch.ones(visual_embeds.shape[:-1], dtype=torch.long) #example
-            >>> visual_attention_mask = torch.ones(visual_embeds.shape[:-1], dtype=torch.float)
+            text = "Who is eating the apple?"
+            inputs = tokenizer(text, return_tensors='pt')
+            visual_embeds = get_visual_embeddings(image).unsqueeze(0)
+            visual_token_type_ids = torch.ones(visual_embeds.shape[:-1], dtype=torch.long)
+            visual_attention_mask = torch.ones(visual_embeds.shape[:-1], dtype=torch.float)
 
-            >>> inputs.update({{
-            ...     "visual_embeds": visual_embeds,
-            ...     "visual_token_type_ids": visual_token_type_ids,
-            ...     "visual_attention_mask": visual_attention_mask
-            ... }})
+            inputs.update({
+                "visual_embeds": visual_embeds,
+                "visual_token_type_ids": visual_token_type_ids,
+                "visual_attention_mask": visual_attention_mask
+            })
 
-            >>> labels = torch.tensor([[0.0,1.0]]).unsqueeze(0)  # Batch size 1, Num labels 2
+            labels = torch.tensor([[0.0,1.0]]).unsqueeze(0)  # Batch size 1, Num labels 2
 
-            >>> outputs = model(**inputs, labels=labels)
-            >>> loss = outputs.loss
-            >>> scores = outputs.logits
+            outputs = model(**inputs, labels=labels)
+            loss = outputs.loss
+            scores = outputs.logits
         """
         return_dict = return_dict if return_dict is not None else self.config.use_return_dict
 
@@ -1285,7 +1297,8 @@ class VisualBertForVisualReasoning(VisualBertPreTrainedModel):
         self.dropout = nn.Dropout(config.hidden_dropout_prob)
         self.cls = nn.Linear(config.hidden_size, config.num_labels)  # 2
 
-        self.init_weights()
+        # Initialize weights and apply final processing
+        self.post_init()
 
     @add_start_docstrings_to_model_forward(VISUAL_BERT_INPUTS_DOCSTRING.format("batch_size, sequence_length"))
     @replace_return_docstrings(output_type=SequenceClassifierOutput, config_class=_CONFIG_FOR_DOC)
@@ -1315,30 +1328,30 @@ class VisualBertForVisualReasoning(VisualBertPreTrainedModel):
 
         Example::
 
-            >>> # Assumption: `get_visual_embeddings(image)` gets the visual embeddings of the image in the batch.
-            >>> from transformers import BertTokenizer, VisualBertForVisualReasoning
-            >>> import torch
+            # Assumption: `get_visual_embeddings(image)` gets the visual embeddings of the image in the batch.
+            from transformers import BertTokenizer, VisualBertForVisualReasoning
+            import torch
 
-            >>> tokenizer = BertTokenizer.from_pretrained('bert-base-uncased')
-            >>> model = VisualBertForVisualReasoning.from_pretrained('uclanlp/visualbert-nlvr2')
+            tokenizer = BertTokenizer.from_pretrained('bert-base-uncased')
+            model = VisualBertForVisualReasoning.from_pretrained('uclanlp/visualbert-nlvr2')
 
-            >>> text = "Who is eating the apple?"
-            >>> inputs = tokenizer(text, return_tensors='pt')
-            >>> visual_embeds = get_visual_embeddings(image).unsqueeze(0)
-            >>> visual_token_type_ids = torch.ones(visual_embeds.shape[:-1], dtype=torch.long) #example
-            >>> visual_attention_mask = torch.ones(visual_embeds.shape[:-1], dtype=torch.float)
+            text = "Who is eating the apple?"
+            inputs = tokenizer(text, return_tensors='pt')
+            visual_embeds = get_visual_embeddings(image).unsqueeze(0)
+            visual_token_type_ids = torch.ones(visual_embeds.shape[:-1], dtype=torch.long)
+            visual_attention_mask = torch.ones(visual_embeds.shape[:-1], dtype=torch.float)
 
-            >>> inputs.update({{
-            ...     "visual_embeds": visual_embeds,
-            ...     "visual_token_type_ids": visual_token_type_ids,
-            ...     "visual_attention_mask": visual_attention_mask
-            ... }})
+            inputs.update({
+                "visual_embeds": visual_embeds,
+                "visual_token_type_ids": visual_token_type_ids,
+                "visual_attention_mask": visual_attention_mask
+            })
 
-            >>> labels = torch.tensor(1).unsqueeze(0)  # Batch size 1, Num choices 2
+            labels = torch.tensor(1).unsqueeze(0)  # Batch size 1, Num choices 2
 
-            >>> outputs = model(**inputs, labels=labels)
-            >>> loss = outputs.loss
-            >>> scores = outputs.logits
+            outputs = model(**inputs, labels=labels)
+            loss = outputs.loss
+            scores = outputs.logits
         """
         return_dict = return_dict if return_dict is not None else self.config.use_return_dict
 
@@ -1441,7 +1454,8 @@ class VisualBertForRegionToPhraseAlignment(VisualBertPreTrainedModel):
         self.cls = VisualBertPreTrainingHeads(config)
         self.attention = VisualBertRegionToPhraseAttention(config)
 
-        self.init_weights()
+        # Initialize weights and apply final processing
+        self.post_init()
 
     @add_start_docstrings_to_model_forward(VISUAL_BERT_INPUTS_DOCSTRING.format("batch_size, sequence_length"))
     @replace_return_docstrings(output_type=SequenceClassifierOutput, config_class=_CONFIG_FOR_DOC)
@@ -1475,32 +1489,32 @@ class VisualBertForRegionToPhraseAlignment(VisualBertPreTrainedModel):
 
         Example::
 
-            >>> # Assumption: `get_visual_embeddings(image)` gets the visual embeddings of the image in the batch.
-            >>> from transformers import BertTokenizer, VisualBertForRegionToPhraseAlignment
-            >>> import torch
+            # Assumption: `get_visual_embeddings(image)` gets the visual embeddings of the image in the batch.
+            from transformers import BertTokenizer, VisualBertForRegionToPhraseAlignment
+            import torch
 
-            >>> tokenizer = BertTokenizer.from_pretrained('bert-base-uncased')
-            >>> model = VisualBertForRegionToPhraseAlignment.from_pretrained('uclanlp/visualbert-vqa-coco-pre')
+            tokenizer = BertTokenizer.from_pretrained('bert-base-uncased')
+            model = VisualBertForRegionToPhraseAlignment.from_pretrained('uclanlp/visualbert-vqa-coco-pre')
 
-            >>> text = "Who is eating the apple?"
-            >>> inputs = tokenizer(text, return_tensors='pt')
-            >>> visual_embeds = get_visual_embeddings(image).unsqueeze(0)
-            >>> visual_token_type_ids = torch.ones(visual_embeds.shape[:-1], dtype=torch.long) #example
-            >>> visual_attention_mask = torch.ones(visual_embeds.shape[:-1], dtype=torch.float)
-            >>> region_to_phrase_position = torch.ones((1, inputs["input_ids"].shape[-1]+visual_embeds.shape[-2]))
+            text = "Who is eating the apple?"
+            inputs = tokenizer(text, return_tensors='pt')
+            visual_embeds = get_visual_embeddings(image).unsqueeze(0)
+            visual_token_type_ids = torch.ones(visual_embeds.shape[:-1], dtype=torch.long)
+            visual_attention_mask = torch.ones(visual_embeds.shape[:-1], dtype=torch.float)
+            region_to_phrase_position = torch.ones((1, inputs["input_ids"].shape[-1]+visual_embeds.shape[-2]))
 
-            >>> inputs.update({{
-            ...     "region_to_phrase_position": region_to_phrase_position,
-            ...     "visual_embeds": visual_embeds,
-            ...     "visual_token_type_ids": visual_token_type_ids,
-            ...     "visual_attention_mask": visual_attention_mask
-            ... }})
+            inputs.update({
+                "region_to_phrase_position": region_to_phrase_position,
+                "visual_embeds": visual_embeds,
+                "visual_token_type_ids": visual_token_type_ids,
+                "visual_attention_mask": visual_attention_mask
+            })
 
-            >>> labels = torch.ones((1, inputs["input_ids"].shape[-1]+visual_embeds.shape[-2], visual_embeds.shape[-2])) # Batch size 1
+            labels = torch.ones((1, inputs["input_ids"].shape[-1]+visual_embeds.shape[-2], visual_embeds.shape[-2])) # Batch size 1
 
-            >>> outputs = model(**inputs, labels=labels)
-            >>> loss = outputs.loss
-            >>> scores = outputs.logits
+            outputs = model(**inputs, labels=labels)
+            loss = outputs.loss
+            scores = outputs.logits
         """
         if region_to_phrase_position is None:
             raise ValueError("`region_to_phrase_position` should not be None when using Flickr Model.")
